#ifndef __tweak_release_h
#define __tweak_release_h
#include "../coordinate.h"
namespace Tweak {
	int balance_extrawave1_i();
	int balance_extrawave2_i();
	int balance_extrawave_random_i();
	int balance_inbetweenwave_i();
	int balance_inbetweenwave_random_i();
	int balance_jumpjets_energy_i();
	int balance_jumpjets_regen_i();
	int balance_max_destruction_i();
	int balance_regen_interval_i();
	int balance_wave_interval_max_i();
	int balance_wave_interval_min_i();
	int balance_weapon_energy_i();
	int balance_weapon_regen_i();
	int balance_win_time_i();
	float blockscale_f();
	float bomb_bombedmat_fact_f();
	float bomb_boxsphere_fact_f();
	float bomb_radius_f();
	float bomb_radius_random_f();
	int bomb_splitlevel_i();
	float bomb_ycheat_f();
	C3 cube_back_color();
	C3 cube_bottom_color();
	C3 cube_front_color();
	C3 cube_left_color();
	C3 cube_right_color();
	C3 cube_top_color();
	int debug_addenemies_i();
	int debug_collisionoff_i();
	int debugdraw_i();
	int debug_flymode_i();
	int debug_hidemap_i();
	float debug_planerot_f();
	int debug_select_level_i();
	int draw_all_i();
	int editmode_i();
	C3f floor_biosphere_pos_C3f();
	C3f floor_bunker_pos_C3f();
	C3f floor_station_pos_C3f();
	C3f floor_tower_pos_C3f();
	C3 fog_color();
	float fog_density_f();
	float fog_end_f();
	float fog_start_f();
	float formation_dist_f();
	int formation_spawn_i();
	int light0_codepos_i();
	C3 light0_color();
	C3f light0_position_C3f();
	int light1_codepos_i();
	C3 light1_color();
	C3f light1_position_C3f();
	int light2_codepos_i();
	C3 light2_color();
	C3f light2_position_C3f();
	C3 material_bombedwall1_color();
	int material_current_i();
	C3 material_slope_bombed1_color();
	C3 material_street1_color();
	C3 material_wall1_color();
	C3 material_wall2_color();
	C3 material_wall3_color();
	C3 material_wall4_color();
	C3 material_window1_color();
	C3 material_window2_color();
	float phys_bombfallconst_f();
	float phys_fallconst_f();
	int plane_1_bombrandom_i();
	float plane_1_speed_f();
	int plane_1_ttl_i();
	int plane_2_bombrandom_i();
	float plane_2_speed_f();
	int plane_2_ttl_i();
	int plane_score_i();
	float plane_y_base_f();
	float plane_y_random_f();
	float player_height_f();
	int player_jump_i();
	float player_max_pos_f();
	float player_min_pos_f();
	float player_speed_f();
	int playmode_i();
	float projectile_forward_f();
	float projectile_speed_f();
	float projectile_starty_f();
	int projectile_ttl_i();
	int radio_duration_default_i();
	C3f radio_pos_C3f();
	C3f radio_scale_C3f();
	int radio_timer_start_i();
	float rot_max_x_f();
	float rot_min_x_f();
	C3 skybox1_color();
	C3 skybox2_color();
	C3 skybox3_color();
	C3 skybox4_color();
	C3 skybox5_color();
	C3 skybox6_color();
	C3 skybox7_color();
	C3 skybox8_color();
	int skybox_draw_i();
	float skybox_size_f();
	C3 ui_destruction_color();
	int ui_gameover_x_i();
	int ui_gameover_y_i();
	C3 ui_jumpjets_color();
	int ui_mouselock_i();
	C3 ui_weapon_color();
}
#endif
